@@ -1,13 +1,33 @@
 import binascii
 import hashlib
 import logging
+import os
 import typing
 from distutils import spawn
 from pathlib import Path
+from urllib.parse import urlparse
+
+import boto3
 
 _LOG = logging.getLogger(__name__)
 
 
+def is_s3_uri(uri):
+    parsed_uri = urlparse(uri)
+    return parsed_uri.scheme == "s3"
+
+
+def get_bucket_key(s3_key):
+    """
+    Return bucket name and key from a s3 key
+    """
+    o = urlparse(s3_key)
+    bucket = o.netloc
+    key = o.path
+    # Remove the leading slash from the prefix/key
+    return bucket, key[1:]
+
+
 def find_exe(name: str):
     """
     Find the location of the given executable.
@@ -39,8 +59,21 @@ def calculate_file_hash(filename, hash_fn=hashlib.sha1, block_size=4096):
     :return: String of hex characters.
     :rtype: str
     """
-    with Path(filename).open("rb") as f:
+    if is_s3_uri(str(filename)):
+        bucket, key = get_bucket_key(filename)
+        try:
+            region_name = os.environ["AWS_DEFAULT_REGION"]
+        except Exception as exp:
+            raise ValueError(
+                "Failed to find AWS_DEFAULT_REGION in the environment variables"
+            ) from exp
+        s3client = boto3.client("s3", region_name=region_name)
+        fileobj = s3client.get_object(Bucket=bucket, Key=key)
+        f = fileobj["Body"].read()
         return calculate_hash(f, hash_fn, block_size)
+    else:
+        with Path(filename).open("rb") as f:
+            return calculate_hash(f, hash_fn, block_size)
 
 
 def calculate_hash(f, hash_fn=hashlib.sha1, block_size=4096):
@@ -91,6 +124,28 @@ class PackageChecksum:
         :type file_path: Path
         :rtype: None
         """
+
+        if is_s3_uri(str(file_path)):
+            try:
+                region_name = os.environ["AWS_DEFAULT_REGION"]
+            except Exception as exp:
+                raise ValueError(
+                    "Failed to find AWS_DEFAULT_REGION in the environment variables"
+                ) from exp
+
+            s3client = boto3.client("s3", region_name)
+            bucket, key = get_bucket_key(file_path)
+            response_obj = s3client.list_objects_v2(Bucket=bucket, Prefix=key)
+            objs = [obj["Key"] for obj in response_obj["Contents"]]
+            if len(objs) > 1:
+                for file_path in objs:
+                    hash_ = self._checksum("s3://{bucket}/{file_path}")
+                    self._append_hash(file_path, hash_)
+            else:
+                hash_ = self._checksum(file_path)
+                self._append_hash(file_path, hash_)
+            return
+
         if file_path.is_dir():
             self.add_files(file_path.iterdir())
         else:
